/* Copyright (c) 2004 David Hammerton - All rights reserved
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <glade/glade.h>

#include "lists.h"
#include "tunesbrowser.h"

static void on_artist_selection_changed(GtkTreeSelection *selection,
                                        gpointer data);
gint artistlist_default_sort(GtkTreeModel *model,
                             GtkTreeIter *a,
                             GtkTreeIter *b,
                             gpointer user_data);

static GtkListStore *artistlist;
static GtkTreeView *artisttreeview;

enum
{
    ALL_SPECIAL_COLUMN,
    ARTIST_COLUMN,
    ARTIST_N_COLUMNS
};

int artistlist_init(GladeXML *xml)
{
    GtkCellRenderer *renderer;
    GtkTreeViewColumn *column;
    GtkTreeSelection *select;

    artistlist = gtk_list_store_new (ARTIST_N_COLUMNS,
                                     G_TYPE_BOOLEAN,
                                     G_TYPE_STRING);
    artisttreeview = GTK_TREE_VIEW(glade_xml_get_widget(xml, "artist"));

    gtk_tree_view_set_model(artisttreeview, GTK_TREE_MODEL(artistlist));

    renderer = gtk_cell_renderer_text_new();
    column = gtk_tree_view_column_new_with_attributes("Artist",
                                                      renderer,
                                                      "text", ARTIST_COLUMN,
                                                      NULL);
    gtk_tree_view_append_column (artisttreeview, column);

    gtk_tree_sortable_set_default_sort_func(GTK_TREE_SORTABLE(artistlist),
                                            artistlist_default_sort,
                                            NULL, NULL);
    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(artistlist),
                                         GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID,
                                         GTK_SORT_ASCENDING);

    select = gtk_tree_view_get_selection(GTK_TREE_VIEW(artisttreeview));
    gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);
    g_signal_connect(G_OBJECT(select), "changed",
                     G_CALLBACK(on_artist_selection_changed),
                     NULL);
    return 1;
}

int artistlist_prepend(char *name, int all)
{
    GtkTreeIter iter;

    gtk_list_store_prepend(artistlist, &iter);

    gtk_list_store_set(artistlist, &iter,
                       ALL_SPECIAL_COLUMN, all ? TRUE : FALSE,
                       ARTIST_COLUMN, name,
                       -1);

    return 1;
}

int artistlist_clear()
{
    gtk_list_store_clear(artistlist);
    return 1;
}

/* callback */
static void on_artist_selection_changed(GtkTreeSelection *selection,
                                        gpointer data)
{
    GtkTreeIter iter;
    GtkTreeModel *model;
    gchar *name;
    gboolean all;

    if (gtk_tree_selection_get_selected(selection, &model, &iter))
    {
        gtk_tree_model_get (model, &iter,
                            ALL_SPECIAL_COLUMN, &all,
                            ARTIST_COLUMN, &name,
                            -1);

        if (all)
            set_daap_artistfilter(NULL);
        else
            set_daap_artistfilter(name);

        g_free(name);
    }
}

/* sort */
gint artistlist_default_sort(GtkTreeModel *model,
                             GtkTreeIter *a,
                             GtkTreeIter *b,
                             gpointer user_data)
{
    gchar *name[2];
    gchar *name_s[2];
    gboolean all[2];

    int ret = 0;

    gtk_tree_model_get(model, a,
                       ALL_SPECIAL_COLUMN, &all[0],
                       ARTIST_COLUMN, &name[0],
                       -1);

    gtk_tree_model_get(model, b,
                       ALL_SPECIAL_COLUMN, &all[1],
                       ARTIST_COLUMN, &name[1],
                       -1);

    if (all[0]) { ret = -1; goto end; }
    if (all[1]) { ret = -1; goto end; }

    name_s[0] = name[0];
    name_s[1] = name[1];
    if (strncasecmp(name_s[0], "the ", 4) == 0)
        name_s[0] += 4;
    if (strncasecmp(name_s[1], "the ", 4) == 0)
        name_s[1] += 4;
    ret = strcasecmp(name_s[0], name_s[1]);
    if (ret != 0) goto end;

end:
    g_free(name[0]);
    g_free(name[1]);

    return ret;
}

