/* Copyright (c) 2004 David Hammerton - All rights reserved
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */


#include <gtk/gtk.h>
#include <glade/glade.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>


#include <gst/gst.h>

#include "lists.h"
#include "tunesbrowser.h"

static GstElement *pipeline_thread, *pipesrc;

static int songstarted = 0;
static int playing = 0;

static void cb_eos(GstElement *elem, gpointer data)
{
    printf("cb_eos(...)\n");
    set_songstatus(STOPPED, -1);
    playing = 0;
    daap_audiocb_finished();
}

static void cb_error (GstElement *gstelement,
                      GstElement *arg1,
                      gpointer arg2,
                      gchar *arg3,
                      gpointer user_data)
{
    printf("an error occured\n");
}

static void cb_iterate(GstBin *bin, gpointer data)
{
    GstClock *clock = gst_bin_get_clock(bin);
    int seconds = gst_clock_get_time(clock) / GST_SECOND;

    seconds = seconds - songstarted;

    if (!playing) seconds = -1;
    set_songpos(seconds);

    return;
}

void audioplayer_init()
{
    GstElement *decoder, *audiosink;

    gst_init(0, NULL);

    pipeline_thread = gst_thread_new ("pipeline");
    gst_bin_set_post_iterate_function(GST_BIN(pipeline_thread),
                                      cb_iterate, NULL);
    pipesrc = gst_element_factory_make ("fdsrc", "pipe_source");
    if (!pipesrc)
    {
        fprintf(stderr, "Failed to load gstreamer source 'fdsrc'.\n");
        goto gst_element_err;
    }
    decoder = gst_element_factory_make ("mad", "decoder");
    if (!decoder)
    {
        fprintf(stderr, "Failed to load gstreamer filter 'mad' (for mp3 decoding)\n");
        goto gst_element_err;
    }
    audiosink = gst_element_factory_make ("osssink", "play_audio");
    if (!audiosink)
        audiosink = gst_element_factory_make ("alsasink", "play_audio");
    if (!audiosink)
        audiosink = gst_element_factory_make ("artsdsink", "play_audio");
    if (!audiosink)
    {
        fprintf(stderr, "Failed to load gstreamer sink - tried all 3 of 'osssink', "
                        "'alsasink' and 'artsdsink'.\n");
        goto gst_element_err;
    }
    gst_bin_add_many (GST_BIN (pipeline_thread), pipesrc, decoder, audiosink, NULL);
    gst_element_link_many (pipesrc, decoder, audiosink, NULL);

    g_signal_connect(G_OBJECT(pipeline_thread), "eos",
                     G_CALLBACK(cb_eos),
                     NULL);
    g_signal_connect(G_OBJECT(pipeline_thread), "error",
                     G_CALLBACK(cb_error),
                     NULL);

    return;

gst_element_err:
    fprintf(stderr, "Couldn't load one of the elements. Please make sure the gstreamer package "
                    "that contains that element is installed on your system. "
                    "(see above for what element may be missing).\n");
    exit(EXIT_FAILURE);
}

void audioplayer_finalize()
{
    g_object_unref(G_OBJECT(pipeline_thread));
}

static void audioplayer_loadpipe(int fd)
{
    g_object_set (G_OBJECT (pipesrc), "fd", fd, NULL);
}

void audioplayer_playpipe(int fd)
{
    playing = 1;
    audioplayer_loadpipe(fd);

    gst_element_set_state (GST_ELEMENT (pipeline_thread), GST_STATE_PLAYING);
    GstClock *clock = gst_bin_get_clock(GST_BIN(pipeline_thread));
    songstarted = gst_clock_get_time(clock) / GST_SECOND;
}

void audioplayer_pause()
{
    gst_element_set_state (GST_ELEMENT (pipeline_thread), GST_STATE_PAUSED);
}

void audioplayer_resume()
{
    gst_element_set_state (GST_ELEMENT (pipeline_thread), GST_STATE_PLAYING);
}

void audioplayer_stop()
{
    printf("audioplayer_stop()\n");
    gst_element_set_state (GST_ELEMENT (pipeline_thread), GST_STATE_READY);
    daap_audiocb_finished();
    playing = 0;
    printf("pusing stopped\n");
    set_songstatus(STOPPED, -1);
}

