/* Copyright (c) 2004 David Hammerton - All rights reserved
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */



#include <gtk/gtk.h>
#include <glade/glade.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/poll.h>
#include <errno.h>

#include <daap/client.h> /* libopendaap */

#include "lists.h"
#include "tunesbrowser.h"

typedef struct hostTAG Shost;
struct hostTAG
{
    DAAP_SClientHost *daap_host;
    DAAP_ClientHost_Database *databases;
    int nDatabases;
    char *sharename;

    Shost *prev;
    Shost *next;
};

Shost *tunesHosts = NULL;

Shost *curHost = NULL;
int curDBID = 0;
static int curSongID = 0;

static int forcedStop = 0;


DAAP_SClient *clientInst;

static int pipefds[2] = {-1, -1}; /* used for download */
static int pipe_shutdownpipe[2] = {-1, -1};
DAAP_Status curstatus = DAAP_STATUS_idle;
void StatusCB(DAAP_SClient *client, DAAP_Status status,
              int pos, void* context)
{
    if (curstatus == DAAP_STATUS_downloading &&
            status == DAAP_STATUS_idle)
    {
        char fake = 1;

        /* close to send EOF */
        close(pipefds[1]);
        pipefds[1] = -1;

        write(pipe_shutdownpipe[1], &fake, sizeof(char));
    }
    else if (status == DAAP_STATUS_downloading)
    {
        /* update % bar or something? */
    }
    else if (status == DAAP_STATUS_hostschanged)
    {
        set_hostsupdated();
        return; /* not a real status */
    }
    curstatus = status;
}

/* for the browser, and primitive sorting */
typedef struct albumTAG album;
struct albumTAG
{
    char *album;
    album *next;
};

typedef struct artistTAG artist;
struct artistTAG
{
    char *artist;
    album *albumhead;
    artist *next;
};

artist *artisthead = NULL;

char *selected_artist;
char *selected_album;

static void free_albums(album *alb)
{
    album *cur = alb;
    while (cur)
    {
        album *next = cur->next;
        if (cur->album) free(cur->album);
        free(cur);
        cur = next;
    }
}
static void free_artists()
{
    artist *cur = artisthead;
    while (cur)
    {
        artist *next = cur->next;
        if (cur->artist) free(cur->artist);
        if (cur->albumhead) free_albums(cur->albumhead);
        free(cur);
        cur = next;
    }
    artisthead = NULL;
}

/* drawing */
static void draw_source()
{
    Shost *cur = tunesHosts;
    sourcelist_clear();

    while (cur)
    {
        sourcelist_append(cur, cur->sharename);
        cur = cur->next;
    }
}

static void draw_artist()
{
    artist *cur = artisthead;
    int count = 0;
    char all_str[] = "All (%i Artists)";
    char all_buf[sizeof(all_str) + 11] = {0};

    artistlist_clear();

    while (cur)
    {
        if (cur->artist[0])
        {
            artistlist_prepend(cur->artist, 0);
            count++;
        }
        cur = cur->next;
    }
    sprintf(all_buf, all_str, count);
    artistlist_prepend(all_buf, 1);
}

static void draw_album()
{
    artist *cur = artisthead;
    int count = 0;
    char all_str[] = "All (%i Albums)";
    char all_buf[sizeof(all_str) + 11] = {0};

    albumlist_clear();

    while (cur)
    {
        album *cur_album;
        if (selected_artist && strcasecmp(selected_artist, cur->artist) != 0)
        {
            cur = cur->next;
            continue;
        }
        cur_album = cur->albumhead;
        while (cur_album)
        {
            if (cur_album->album[0])
            {
                albumlist_prepend(cur_album->album, 0);
                count++;
            }

            cur_album = cur_album->next;
        }
        cur = cur->next;
    }
    sprintf(all_buf, all_str, count);
    albumlist_prepend(all_buf, 1);
}

/* database */
static void updateDatabases(Shost *host)
{
    int size;
    size = DAAP_ClientHost_GetDatabases(host->daap_host,
                                        NULL, NULL, 0);

    if (host->databases) free(host->databases);
    host->databases = malloc(size);
    DAAP_ClientHost_GetDatabases(host->daap_host,
                                 host->databases,
                                 &(host->nDatabases),
                                 size);
}

int addDAAPEnumCB(DAAP_SClient *client, DAAP_SClientHost *host, void *ctx)
{
    Shost *newhost = malloc(sizeof(Shost));

    DAAP_ClientHost_AddRef(host);

    newhost->prev = NULL;
    newhost->next = tunesHosts;
    newhost->daap_host = host;

    newhost->databases = NULL;
    newhost->nDatabases = 0;

    tunesHosts = newhost;

    return 1;
}

void init_daap()
{
    clientInst = DAAP_Client_Create(StatusCB, NULL);
}

void init_daap_sources()
{
    /* FIXME should discover */
    Shost *curhost;

    DAAP_Client_EnumerateHosts(clientInst,
            addDAAPEnumCB, NULL);

    curhost = tunesHosts;

    while (curhost)
    {
        char *buf;
        int size;

        size = DAAP_ClientHost_GetSharename(curhost->daap_host,
                                            NULL, 0);
        buf = malloc(size);
        size = DAAP_ClientHost_GetSharename(curhost->daap_host,
                                            buf, size);
        curhost->sharename = buf;

        curhost = curhost->next;
    }

    draw_source();
}

void finalize_daap_sources()
{
    Shost *cur = tunesHosts;
    while (cur)
    {
        Shost *next = cur->next;
        free(cur->sharename);
        if (cur->databases) free(cur->databases);
        DAAP_ClientHost_Release(cur->daap_host);
        free(cur);
        cur = next;
    }
    tunesHosts = NULL;
}


/* browser */
static void artistalbumview_start()
{
    free_artists();
}
static void artistalbumview_add(char *artist_s, char *album_s)
{
    artist *cur_artist = artisthead;
    album *cur_album = NULL;
    while (cur_artist)
    {
        if (strcasecmp(cur_artist->artist, artist_s) == 0)
            break;
        cur_artist = cur_artist->next;
    }
    if (!cur_artist)
    {
        artist *newartist = malloc(sizeof(artist));

        newartist->artist = malloc(strlen(artist_s) +1);
        strcpy(newartist->artist, artist_s);

        newartist->albumhead = NULL;

        newartist->next = artisthead;
        artisthead = newartist;

        cur_artist = newartist;
    }
    cur_album = cur_artist->albumhead;
    while (cur_album)
    {
        if (strcasecmp(cur_album->album, album_s) == 0)
            break;
        cur_album = cur_album->next;
    }
    if (!cur_album)
    {
        album *newalbum = malloc(sizeof(album));

        newalbum->album = malloc(strlen(album_s) + 1);
        strcpy(newalbum->album, album_s);

        newalbum->next = cur_artist->albumhead;
        cur_artist->albumhead = newalbum;

        cur_album = newalbum;
    }
}

/* songs */
/* cache it */
static DAAP_ClientHost_DatabaseItem *currentSongItems = NULL;
static int currentSongItemCount = 0;

/* find a song in the current song items (by id) - returns an index, -1 on error */
static int findSongByID(int id)
{
    int i;
    for (i = 0; i < currentSongItemCount; i++)
    {
        if (currentSongItems[i].id == id)
            return i;
    }
    return -1;
}

static int compare_songitems(const void *pva, const void *pvb)
{
    DAAP_ClientHost_DatabaseItem *a = (DAAP_ClientHost_DatabaseItem*)pva;
    DAAP_ClientHost_DatabaseItem *b = (DAAP_ClientHost_DatabaseItem*)pvb;

    int ret;

    char *artist[2], *album[2], *name[2];
    int track_i[2];

    /* compare artist */
    artist[0] = a->songartist;
    artist[1] = b->songartist;
    if (!artist[0][0]) return -1;
    if (!artist[1][0]) return 1;
    if (strncasecmp(artist[0], "the ", 4) == 0)
        artist[0] += 4;
    if (strncasecmp(artist[1], "the ", 4) == 0)
        artist[1] += 4;
    ret = strcasecmp(artist[0], artist[1]);
    if (ret != 0) return -ret;

    /* compare album */
    album[0] = a->songalbum;
    album[1] = b->songalbum;
    if (!album[0][0]) return -1;
    if (!album[1][0]) return 1;
    if (strncasecmp(album[0], "the ", 4) == 0)
        album[0] += 4;
    if (strncasecmp(album[1], "the ", 4) == 0)
        album[1] += 4;
    ret = strcasecmp(album[0], album[1]);
    if (ret != 0) return -ret;

    track_i[0] = a->songtracknumber;
    track_i[1] = b->songtracknumber;
    ret = (track_i[0] < track_i[1] ? -1 :
            (track_i[0] > track_i[1] ? 1 : 0));
    if (ret != 0) return -ret;

    name[0] = a->itemname;
    name[1] = b->itemname;
    if (!name[0][0]) return -1;
    if (!name[1][0]) return 1;
    if (strncasecmp(name[0], "the ", 4) == 0)
        name[0] += 4;
    if (strncasecmp(name[1], "the ", 4) == 0)
        name[1] += 4;
    ret = strcasecmp(name[0], name[1]);
    if (ret != 0) return -ret;

    return 0;
}

/* gets the song list, adds them to the database, and sorts them */
static void getSongItems(Shost *host, int id)
{
    int i;
    int size;

    if (currentSongItems) free(currentSongItems);

    size = DAAP_ClientHost_GetDatabaseItems(host->daap_host, id,
                                            NULL, NULL, 0);
    currentSongItems = (DAAP_ClientHost_DatabaseItem*)malloc(size);
    DAAP_ClientHost_GetDatabaseItems(host->daap_host, id,
                                     currentSongItems,
                                     &currentSongItemCount,
                                     size);

    for (i = 0; i < currentSongItemCount; i++)
    {
        artistalbumview_add(currentSongItems[i].songartist,
                            currentSongItems[i].songalbum);
    }
    qsort(currentSongItems, currentSongItemCount,
          sizeof(DAAP_ClientHost_DatabaseItem),
          compare_songitems);
}

static void draw_songs()
{
    int i;

    songlist_clear();

    for (i = 0; i < currentSongItemCount; i++)
    {
        char time[11] = {0};
        char track[11] = {0};
        char year[11] = {0};

        if (selected_artist &&
                strcasecmp(selected_artist, currentSongItems[i].songartist) != 0)
            continue;
        if (selected_album &&
                strcasecmp(selected_album, currentSongItems[i].songalbum) != 0)
            continue;

        int minutes, seconds;

        seconds = (currentSongItems[i].songtime) / 1000;
        minutes = seconds / 60;
        seconds -= minutes * 60;

        snprintf(time, 10, "%i:%02i",
                           minutes, seconds);
        if (currentSongItems[i].songtracknumber)
            snprintf(track, 10, "%i", currentSongItems[i].songtracknumber);
        if (currentSongItems[i].songyear)
            snprintf(year, 10, "%i", currentSongItems[i].songyear);

        songlist_append(currentSongItems[i].id,
                        currentSongItems[i].itemname,
                        currentSongItems[i].songartist,
                        currentSongItems[i].songalbum,
                        time, track, year
                        );
    }
}

void set_daap_sources(void *host)
{
    int id;

    if (curHost)
    {
        DAAP_ClientHost_Disconnect(curHost->daap_host);
        DAAP_ClientHost_Release(curHost->daap_host);
    }

    curHost = (Shost *)host;

    DAAP_ClientHost_AddRef(curHost->daap_host);
    DAAP_ClientHost_Connect(curHost->daap_host);

    artistalbumview_start();

    updateDatabases(curHost);
    id = curHost->databases[0].id;

    if (curHost->nDatabases > 1)
    {
        int i;
        printf("not really sure what to do with multiple databases "
               "(never seen it myself). The following databases will "
               "be invisible\n");
        for (i = 1; i < curHost->nDatabases; i++)
        {
            printf("invisible database: '%s'\n",
                   curHost->databases[i].name);
        }
    }

    curDBID = id;

    getSongItems(curHost, id);
    draw_songs();
    draw_artist();
    draw_album();
}

/* filter */
void set_daap_artistfilter(char *name)
{
    if (selected_artist)
    {
        free(selected_artist);
        selected_artist = NULL;
    }
    if (name)
    {
        selected_artist = malloc(strlen(name) + 1);
        strcpy(selected_artist, name);
    }
    draw_album();
    set_daap_albumfilter(NULL);
    draw_songs();
}

void set_daap_albumfilter(char *name)
{
    if (selected_album)
    {
        free(selected_album);
        selected_album = NULL;
    }
    if (name)
    {
        selected_album = malloc(strlen(name) + 1);
        strcpy(selected_album, name);
    }
    draw_songs();
}

char *get_song_display_str_withalloc(int id)
{
    int songindex = findSongByID(id);

    if (!songindex) return NULL;

    char *buf;
    buf = malloc(strlen(currentSongItems[songindex].songartist) + strlen(" - ")
                 + strlen(currentSongItems[songindex].itemname) + 1);

    strcpy(buf, currentSongItems[songindex].songartist);
    strcat(buf, " - ");
    strcat(buf, currentSongItems[songindex].itemname);

    return buf;
}

/* song */
void play_daap_song(int id)
{
    int songindex = findSongByID(id);

    curSongID = id;

    if (!songindex) return;

    if (pipefds[0] != -1 || pipefds[1] != -1)
    {
        char fake;
        struct pollfd ufd;

        forcedStop = 1;

        /* FIXME need a better way to shut it down */
        /* wait for everything to shut down... */
        DAAP_ClientHost_AsyncStop(curHost->daap_host);

        read(pipe_shutdownpipe[0], &fake, sizeof(char));
        read(pipe_shutdownpipe[0], &fake, sizeof(char));

        audioplayer_stop();

    }
    if (pipe_shutdownpipe[0] != -1)
        close(pipe_shutdownpipe[0]);
    if (pipe_shutdownpipe[1] != -1)
        close(pipe_shutdownpipe[1]);
    pipe(pipefds);
    pipe(pipe_shutdownpipe);

    if (DAAP_ClientHost_AsyncGetAudioFile(curHost->daap_host,
                                          curDBID, id,
                                          pipefds[1]))
        goto fail;

    set_songstatus(PLAYING, id);

    audioplayer_playpipe(pipefds[0]);

    forcedStop = 0;

    return;

fail:
    close(pipefds[0]);
    close(pipefds[1]);
}

void pause_daap_song()
{
    audioplayer_pause();

    set_songstatus(PAUSED, curSongID);
}

void resume_daap_song()
{
    audioplayer_resume();

    set_songstatus(PLAYING, curSongID);
}

void daap_playnext_ifstillvisible()
{
    int index = findSongByID(curSongID);
    int i;

    if (forcedStop) return;

    /* if its not visible, don't try and play the next one */
    if (selected_artist &&
            strcasecmp(selected_artist, currentSongItems[index].songartist) != 0)
        return;
    if (selected_album &&
            strcasecmp(selected_album, currentSongItems[index].songalbum) != 0)
        return;

    for (i = index-1; i >= 0; i--)
    {
        if (selected_artist &&
                strcasecmp(selected_artist, currentSongItems[i].songartist) != 0)
            return;
        if (selected_album &&
                strcasecmp(selected_album, currentSongItems[i].songalbum) != 0)
            return;
        play_daap_song(currentSongItems[i].id);
        return;
    }
}

void daap_audiocb_finished()
{
    char fake = 0;
    /* close the pipe */
    close(pipefds[0]);
    pipefds[0] = -1;

    write(pipe_shutdownpipe[1], &fake, sizeof(char));

    daap_playnext_ifstillvisible();
}


