/* Copyright (c) 2004 David Hammerton - All rights reserved
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */



#include <stdio.h>
#include <gtk/gtk.h>
#include <glade/glade.h>

#include "lists.h"
#include <string.h>
#include <stdlib.h>

static GtkListStore *songlist;
static GtkTreeView *songtreeview;

enum
{
    COL_ID,
    COL_ITEMNAME,
    COL_SONGARTIST,
    COL_SONGALBUM,
    COL_SONGTIME,
    COL_SONGTRACK,
    COL_SONGYEAR,
    N_COLUMNS
};

static void on_song_row_activated(GtkTreeView *treeview,
                                  GtkTreePath *arg1,
                                  GtkTreeViewColumn *arg2,
                                  gpointer user_data);

/* sorts */
gint songlist_default_sort(GtkTreeModel *model,
                           GtkTreeIter *a,
                           GtkTreeIter *b,
                           gpointer user_data);

int songlist_init(GladeXML *xml)
{
    GtkCellRenderer *renderer;
    GtkTreeViewColumn *column;
    GtkTreeSelection *select;

    songlist = gtk_list_store_new (N_COLUMNS,
                                    G_TYPE_INT, /* id */
                                    G_TYPE_STRING, /* itemname */
                                    G_TYPE_STRING, /* artist */
                                    G_TYPE_STRING, /* album */
                                    G_TYPE_STRING, /* time */
                                    G_TYPE_STRING, /* track */
                                    G_TYPE_STRING /* year */
                                    );
    songtreeview = GTK_TREE_VIEW(glade_xml_get_widget(xml, "songs"));

    gtk_tree_view_set_model(songtreeview, GTK_TREE_MODEL(songlist));

    renderer = gtk_cell_renderer_text_new();
    column = gtk_tree_view_column_new_with_attributes("Song Name",
                                                      renderer,
                                                      "text", COL_ITEMNAME,
                                                      NULL);
    gtk_tree_view_column_set_resizable(column, TRUE);
    gtk_tree_view_append_column (songtreeview, column);

    renderer = gtk_cell_renderer_text_new();
    column = gtk_tree_view_column_new_with_attributes("Time",
                                                      renderer,
                                                      "text", COL_SONGTIME,
                                                      NULL);
    gtk_tree_view_column_set_resizable(column, TRUE);
    gtk_tree_view_append_column (songtreeview, column);

    renderer = gtk_cell_renderer_text_new();
    column = gtk_tree_view_column_new_with_attributes("Artist",
                                                      renderer,
                                                      "text", COL_SONGARTIST,
                                                      NULL);
    gtk_tree_view_column_set_resizable(column, TRUE);
    gtk_tree_view_append_column (songtreeview, column);

    renderer = gtk_cell_renderer_text_new();
    column = gtk_tree_view_column_new_with_attributes("Track #",
                                                      renderer,
                                                      "text", COL_SONGTRACK,
                                                      NULL);
    gtk_tree_view_column_set_resizable(column, TRUE);
    gtk_tree_view_append_column (songtreeview, column);

    renderer = gtk_cell_renderer_text_new();
    column = gtk_tree_view_column_new_with_attributes("Album",
                                                      renderer,
                                                      "text", COL_SONGALBUM,
                                                      NULL);
    gtk_tree_view_column_set_resizable(column, TRUE);
    gtk_tree_view_append_column (songtreeview, column);

    renderer = gtk_cell_renderer_text_new();
    column = gtk_tree_view_column_new_with_attributes("Year",
                                                      renderer,
                                                      "text", COL_SONGYEAR,
                                                      NULL);
    gtk_tree_view_column_set_resizable(column, TRUE);
    gtk_tree_view_append_column (songtreeview, column);

    gtk_tree_sortable_set_default_sort_func(GTK_TREE_SORTABLE(songlist),
                                            songlist_default_sort,
                                            NULL, NULL);
    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(songlist),
                                         GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID,
                                         GTK_SORT_ASCENDING);

    select = gtk_tree_view_get_selection(GTK_TREE_VIEW(songtreeview));
    gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);

    g_signal_connect(G_OBJECT(songtreeview), "row-activated",
                     G_CALLBACK(on_song_row_activated),
                     NULL);

    return 1;
}

int songlist_append(int id, char *name, char *artist,
                    char *album, char *time, char *track,
                    char *year)
{
    GtkTreeIter iter;

    gtk_list_store_append(songlist, &iter);

    gtk_list_store_set(songlist, &iter,
                       COL_ID, id,
                       COL_ITEMNAME, name,
                       COL_SONGARTIST, artist,
                       COL_SONGALBUM, album,
                       COL_SONGTIME, time,
                       COL_SONGTRACK, track,
                       COL_SONGYEAR, year,
                       -1);

    return 1;
}

int songlist_clear()
{
    gtk_list_store_clear(songlist);
    return 1;
}

int songlist_get_selected_id()
{
    gint id;
    GtkTreeSelection *selection =
        gtk_tree_view_get_selection(songtreeview);
    GtkTreeIter iter;
    GtkTreeModel *model;

    if (gtk_tree_selection_get_selected(selection, &model, &iter))
    {
        gtk_tree_model_get(model, &iter,
                           COL_ID, &id,
                           -1);

        return id;
    }
}

/* callbacks */
static void on_song_row_activated(GtkTreeView *treeview,
                                  GtkTreePath *arg1,
                                  GtkTreeViewColumn *arg2,
                                  gpointer user_data)
{
    play_daap_song(songlist_get_selected_id());
}

/* sort funcs */
gint songlist_default_sort(GtkTreeModel *model,
                           GtkTreeIter *a,
                           GtkTreeIter *b,
                           gpointer user_data)
{
    gchar *artist[2];
    gchar *artist_s[2];
    gchar *album[2];
    gchar *album_s[2];
    gchar *track[2];
    gchar *name[2];
    gchar *name_s[2];

    int track_i[2];

    int ret = 0;

    return 0;

    gtk_tree_model_get(model, a,
                       COL_SONGARTIST, &artist[0],
                       COL_SONGALBUM, &album[0],
                       COL_SONGTRACK, &track[0],
                       COL_ITEMNAME, &name[0],
                       -1);

    gtk_tree_model_get(model, b,
                       COL_SONGARTIST, &artist[1],
                       COL_SONGALBUM, &album[1],
                       COL_SONGTRACK, &track[1],
                       COL_ITEMNAME, &name[1],
                       -1);

    artist_s[0] = artist[0];
    artist_s[1] = artist[1];
    if (!artist_s[0][0]) { ret = 1; goto end; }
    if (!artist_s[1][0]) { ret = -1; goto end; }
    if (strncasecmp(artist_s[0], "the ", 4) == 0)
        artist_s[0] += 4;
    if (strncasecmp(artist_s[1], "the ", 4) == 0)
        artist_s[1] += 4;
    ret = strcasecmp(artist_s[0], artist_s[1]);
    if (ret != 0) goto end;

    album_s[0] = album[0];
    album_s[1] = album[1];
    if (!album_s[0][0]) { ret = 1; goto end; }
    if (!album_s[1][0]) { ret = -1; goto end; }
    if (strncasecmp(album_s[0], "the ", 4) == 0)
        album_s[0] += 4;
    if (strncasecmp(album_s[1], "the ", 4) == 0)
        album_s[1] += 4;
    ret = strcasecmp(album_s[0], album_s[1]);
    if (ret != 0) goto end;

    track_i[0] = atoi(track[0]);
    track_i[1] = atoi(track[1]);
    ret = (track_i[0] < track_i[1] ? -1 :
            (track_i[0] > track_i[1] ? 1 : 0));
    if (ret != 0) goto end;

    name_s[0] = name[0];
    name_s[1] = name[1];
    if (!name_s[0][0]) { ret = 1; goto end; }
    if (!name_s[1][0]) { ret = -1; goto end; }
    if (strncasecmp(name_s[0], "the ", 4) == 0)
        name_s[0] += 4;
    if (strncasecmp(name_s[1], "the ", 4) == 0)
        name_s[1] += 4;
    ret = strcasecmp(name_s[0], name_s[1]);
    if (ret != 0) goto end;

end:
    g_free(artist[0]);
    g_free(album[0]);
    g_free(track[0]);
    g_free(name[0]);
    g_free(artist[1]);
    g_free(album[1]);
    g_free(track[1]);
    g_free(name[1]);

    return ret;
}

