/* Copyright (c) 2004 David Hammerton - All rights reserved
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */


#include <stdio.h>
#include <gtk/gtk.h>
#include <glade/glade.h>

#include "lists.h"
#include "tunesbrowser.h"

static void on_source_selection_changed(GtkTreeSelection *selection,
                                        gpointer data);

static GtkListStore *sourcelist;
static GtkTreeView *sourcetreeview;

enum
{
    SOURCE_COLUMN,
    HOST_COLUMN,
    SOURCE_N_COLUMNS
};

int sourcelist_init(GladeXML *xml)
{
    GtkCellRenderer *renderer;
    GtkTreeViewColumn *column;
    GtkTreeSelection *select;

    sourcelist = gtk_list_store_new (SOURCE_N_COLUMNS,
                                     G_TYPE_STRING,
                                     G_TYPE_POINTER);
    sourcetreeview = GTK_TREE_VIEW(glade_xml_get_widget(xml, "source"));

    gtk_tree_view_set_model(sourcetreeview, GTK_TREE_MODEL(sourcelist));

    renderer = gtk_cell_renderer_text_new();
    column = gtk_tree_view_column_new_with_attributes("Source",
                                                      renderer,
                                                      "text", SOURCE_COLUMN,
                                                      NULL);
    gtk_tree_view_append_column (sourcetreeview, column);

    select = gtk_tree_view_get_selection (GTK_TREE_VIEW (sourcetreeview));
    gtk_tree_selection_set_mode (select, GTK_SELECTION_SINGLE);
    g_signal_connect (G_OBJECT (select), "changed",
                      G_CALLBACK (on_source_selection_changed),
                      NULL);

    return 1;
}

int sourcelist_append(void *host, char *name)
{
    GtkTreeIter iter;

    gtk_list_store_append(sourcelist, &iter);

    gtk_list_store_set(sourcelist, &iter,
                       SOURCE_COLUMN, name,
                       HOST_COLUMN, host,
                       -1);

    return 1;
}

int sourcelist_clear()
{
    gtk_list_store_clear(sourcelist);
    return 1;
}

/* callback */
static void on_source_selection_changed(GtkTreeSelection *selection,
                                        gpointer data)
{
    GtkTreeIter iter;
    GtkTreeModel *model;
    gpointer *host;

    if (gtk_tree_selection_get_selected(selection, &model, &iter))
    {
        gtk_tree_model_get (model, &iter,
                            HOST_COLUMN, &host,
                            -1);

        set_daap_sources(host);
    }

}



